﻿// #AdvancedSearch.jsx

/*
v.06072022  

=== Текст буфера в строке поиска ===    
Поиск в вёрстке текста, помещённого в буфер, несколько слов, или несколько абзацев, не важно.

Смена пробелов на шпации, изменение вида кавычек, удаление повторяющихся пробелов, смена дефиса на минус или тире  — 
главная причина, что отстутствует лёгкость поиска в индизайне текста, взятого в буфер в редакторе Word или в PDF-файле.

При подготовке текста к вёрстке убрали повторяющиеся пробелы, дефис в последовательности символов "пробел-дефис-пробел" между словами заменили на тире или минус, изменили вид кавычек.
Какие-то пробелы стали шпациями. Это, конечно, хорошо. Но пропало удобство поиска текста. Невозможно взять в буфер текст в редакторе Word и сразу найти его в вёрстке.

Данный скрипт позволяет искать в вёрстке взятый в буфер текст, закрывая глаза на следующие случаи:
= повторяющиеся пробелы, и то, что они могли стать шпациями; 
= окруженные шпициями дефис/минус/тире считаются одинаковыми знаками;
= если кавычка есть, то она может быть любой.
= различие в числе повторяющихся переводов строки не считается расхождением вёрстки и исходного текста.

Знаки пунктуации — запятая, точка, и пр. должны быть такими, как в переданном на вёрстку тексте. Тут вариантов быть не может.

Это решение позволяет брать в буфер сырой текст из редактора Word или блокнота, или из PDF-файла, и искать его в вёрстке.
Причём только брать, дальше — подготовку текста к такому поиску, без контроля шпаций, вариантов кавычек, что используется в качестве тире, 
и помещение этой строки в поле "Найти текст:" в панели GREP сделает скрипт.

Текст буфера изменяется, чтобы шпации на месте пробелов, и разный вид кавычек не были препятствием для поиска.
- в тексте из буфера пробелы заменяются на \h — grep-код поиска шпаций и табуляций.
-  вид кавычек тоже может быть разным, на их месте ставится точка как код любого знака.
- подготовленный для поиска текст помещается в поле grep-поиска.

Поиск выполняется от точки вставки курсора и до конца материала, если курсор в тексте, или в выделенной области материала.
Найденный текст отображается посередине экрана в масштабе 150%.

Скрипт для версий CS6+.

Инструкция с несколько другими акцентами на использовании этой программы есть в сопроводительном PDF-файле.

------------------------------------------------
dotextok.ru  |   dotextok@gmail.com

*/
var debug = false; // эта переменная должна быть в состоянии false. Переключение в true -- только для получения информации об ошибке, чтобы сообщить в письме мне. Может быть, сразу удастся понять, в чём дело. Обычно в преочессе обмена сообщениями снимались все вопросы.
var showPDF = true; // если true, то после сообщения, как определять область поиска, будет выведен PDF-файл Advanced Search Info.pdf

app.scriptPreferences.userInteractionLevel = UserInteractionLevels.interactWithAll; // см. http://adobeindesign.ru/2008/10/24/restore-ui/
var programTitul = "Текст буфера в строке поиска";
if(app.documents.length == 0) {
    alert("Нет открытого документа.");	
    exit();    
    }
if (parseInt (app.version) < 8) {
	alert ("Этот скрипт предназначен для использования в InDesign CS6+.");
    exit();
    }
var sel = app.selection[0];
if ((sel == null) || (sel.constructor.name != "InsertionPoint" && sel.constructor.name != "Paragraph" && sel.constructor.name != "Text" && sel.constructor.name != "TextColumn" && sel.constructor.name != "TextStyleRange")) { // == null
    alert("Скрипт ищет в вёрстке сохранённый в буфере текст. Если часть материала выделена, то поиск в выделенной области. Если курсор в тексте, то поиск в материале, начиная от точки вставки курсора до конца статьи.",programTitul);
    if (showPDF) { // showPDF
        app.pdfExportPreferences.properties.viewPDF = true;   
        var myScriptFile = getScriptPath();
        var myScriptFolder = decodeURI(myScriptFile.path);  
        var pdfName = "Advanced Search Info.pdf";
        var infoFile = File(decodeURI(myScriptFolder + "/" + pdfName));
        if (!infoFile.exists) { alert("Не найден PDF-файл с описанием этой программы. Он называется '" + pdfName + "' и должен быть в той же папке, где скрипт.",programTitul); }
        else { infoFile.execute(); }
        } // showPDF
    exit(); 
    } // == null
var story = sel.parentStory;    
var space;
if (sel.constructor.name == "InsertionPoint") {
    var startIndex = sel.index;
    var endIndex = story.length-1;
    story.characters.itemByRange(startIndex, endIndex).select();
    space = app.selection[0];
    }
else space = sel;
var startPosition = space.characters[0].index;
var usedZoom = app.activeWindow.zoomPercentage;
var theFrame = app.activeWindow.activePage.textFrames.add({strokeWeight:0});
if (theFrame.strokeWeight != 0) theFrame.strokeWeight = 0;
theFrame.geometricBounds = ["-45mm",0,"-40mm","140mm"];
theFrame.texts[0].appliedParagraphStyle = app.activeDocument.paragraphStyles[0];
theFrame.texts[0].clearOverrides();
theFrame.textFramePreferences.ignoreWrap = true;
theFrame.textFramePreferences.verticalJustification = VerticalJustification.TOP_ALIGN; 
theFrame.texts[0].alignToBaseline = false;
theFrame.texts[0].pointSize = 3;
theFrame.bringToFront();
theFrame.texts[0].contents = "";
theFrame.insertionPoints[0].select();
var erMes = "Похоже, что в буфере нет текста. Или содержимое буфера текстом не является.";
try {
    app.paste();
    }
catch (e) {
     theFrame.remove();
    alert(erMes + " [1]", programTitul);
    exit();      
    }
if (theFrame.texts[0].length == 0) {
    theFrame.remove();
    alert(erMes + " [2]", programTitul);
    exit();    
    }
///
var notUse = 0;
var newString = "";
// иногда, по непонятной причине, в содержимом буфера после копирования текста из некоторых pdf-файлов сначала идёт перевод строки и пробел, а потом уже тот текст, что был выделен.
for (var i = 0; i < theFrame.parentStory.texts[0].length; i++) { // i++
    if (theFrame.parentStory.texts[0].characters[i].contents == " ") {
        notUse = notUse+1; continue; 
        }
    if (theFrame.parentStory.texts[0].characters[i].contents == "\r") {
        notUse = notUse+1; continue; 
        }    
    if (theFrame.parentStory.texts[0].characters[i].contents == "\n") {
        notUse = notUse+1; continue; 
        }       
    break;
    } // i++
if (notUse > 0) {
    newString = theFrame.parentStory.texts[0].contents.substring(notUse, theFrame.parentStory.texts[0].length);
    theFrame.parentStory.texts[0].contents = newString;
    }
var typoQuotes = app.activeDocument.textPreferences.typographersQuotes;
app.activeDocument.textPreferences.typographersQuotes = false;
////
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findGrepPreferences.findWhat = "\\x{003F}"; 
app.changeGrepPreferences.changeTo = "\\?";
theFrame.parentStory.texts[0].changeGrep();
///
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findGrepPreferences.findWhat = "\\"; 
app.changeGrepPreferences.changeTo = "\\\\"; // это не ошибка, так оформляется в скрипте поиск одиночной наклонной черты. В окне Найти/Заменить для поиска одной наклонной черты их будет стоять две. Объяснение: "экранирование экранирующего символа" -- корявая калька с английского.
theFrame.parentStory.texts[0].changeGrep();
///
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findGrepPreferences.findWhat = "\\)"; 
app.changeGrepPreferences.changeTo = "\\)";
theFrame.parentStory.texts[0].changeGrep();
///
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findGrepPreferences.findWhat = "\\("; 
app.changeGrepPreferences.changeTo = "\\(";
theFrame.parentStory.texts[0].changeGrep();
///
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findGrepPreferences.findWhat = "\\]"; 
app.changeGrepPreferences.changeTo = "\\]";
theFrame.parentStory.texts[0].changeGrep();
///
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findGrepPreferences.findWhat = "\\["; 
app.changeGrepPreferences.changeTo = "\\[";
theFrame.parentStory.texts[0].changeGrep();
///
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findGrepPreferences.findWhat = "\\r"; 
app.changeGrepPreferences.changeTo = "\\r*";
theFrame.parentStory.texts[0].changeGrep();
///
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findGrepPreferences.findWhat = "\\x{00AD}"; 
app.changeGrepPreferences.changeTo = "\\x{00AD}*"; // для обработки переносов в тексте, взятом из PDF
theFrame.parentStory.texts[0].changeGrep();
///
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findGrepPreferences.findWhat = "(?<=[[:blank:]])[-–—]+(?=\\s)"; // плюс --- для учёта случая, что в вордовском файле тире оформлено как "пробел дефис дефис пробел". Предполагается, что в индизайне текст уже приведён в порядок, прежде чем будет использован этот скрипт
app.changeGrepPreferences.changeTo = "[-–—]";
theFrame.parentStory.texts[0].changeGrep();
///
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findGrepPreferences.findWhat = "[[:blank:]]+"; 
app.changeGrepPreferences.changeTo = "\\h+";
theFrame.parentStory.texts[0].changeGrep();
///
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findGrepPreferences.findWhat = "[«»„“”\"\'‚‹›‘’]"; 
//app.changeGrepPreferences.changeTo = '\"';  // 05.07.2022  обнаружилось странное поведение при обработке текста этими запросами: если в одном запросе есть обработка кавычки и тире, то запрос не работает.
app.changeGrepPreferences.changeTo = "."; // поэтому для этой задачи поиска на месте любой кавычки будет точка -- по правилам grep это любой символ. Для такого поиска это упрощение вполне приемлемо.
theFrame.parentStory.texts[0].changeGrep();
///
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findGrepPreferences.findWhat = "([\\l\\d])([-‑])([\\l\\d])"; // вариант дефиса между буквами/цифрами -- обычный или неразрывный
app.changeGrepPreferences.changeTo = "$1[-‑]$3"; 
theFrame.parentStory.texts[0].changeGrep();
///
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findChangeGrepOptions.includeFootnotes = true;
newString = theFrame.parentStory.texts[0].contents;
app.findGrepPreferences.findWhat = theFrame.parentStory.texts[0].contents;    
var found = space.findGrep();
theFrame.remove();
if (!debug) {
    app.findGrepPreferences = null; 
    app.changeGrepPreferences = null;
    }
if (found.length == 0) {
    story.insertionPoints[startPosition].select();
    app.activeWindow.activePage = app.selection[0].parentTextFrames[0].parentPage; 
    app.activeWindow.zoomPercentage = usedZoom;    
    if (debug) { $.writeln("\r\r-----------------\r" + newString); newString = "\r\r" + newString; } else newString = "";
    alert("Текст не найден." + newString, programTitul);
    app.activeDocument.textPreferences.typographersQuotes = typoQuotes;    
    app.documents[0].select(NothingEnum.nothing);
    exit();
    }
found[0].select();
app.activeWindow.activePage = app.selection[0].parentTextFrames[0].parentPage; 
app.activeWindow.zoomPercentage = 150;
app.activeDocument.textPreferences.typographersQuotes = typoQuotes;
exit();
///////
function getScriptPath() { //  //getScriptPath()
	try{
		return app.activeScript;
	}
	catch(e){
		return File(e.fileName);
	}
} //getScriptPath()